# support : "title", "rich_text", "number", "select", "multi_select", "checkbox", "url", "email", "phone_number"

class Properties:
    def __init__(self):
        self.result = {}

    def set_title(self, col, text=None):
        if text:
            text = [{"text": {"content": text}}]
        else:
            text = {}
        self.result.update({col: {"title": text}})

    def set_rich_text(self, col, text=None):
        if text:
            text = [{"text": {"content": text}}]
        else:
            text = {}
        self.result.update({col: {"rich_text": text}})

    def set_number(self, col, text=None):
        if text:
            text = int(text)
        else:
            text = {}
        self.result.update({col: {"number": text}})

    def set_select(self, col, text=None):
        if text:
            text = {"name": text}
        else:
            text = {}
        self.result.update({col: {"select": text}})

    def set_multi_select(self, col, text_list=None):
        if text_list:
            data = []
            for i in text_list:
                data.append({"name": i})
        else:
            data = {}

        self.result.update({col: {"multi_select": data}})

    def set_checkbox(self, col, text=None):
        if not text:
            text = {}
        self.result.update({col: {"checkbox": text}})

    def set_url(self, col, text=None):
        if not text:
            text = {}
        self.result.update({col: {"url": text}})

    def set_email(self, col, text=None):
        if not text:
            text = {}
        self.result.update({col: {"email": text}})

    def set_phone_number(self, col, text=None):
        if not text:
            text = {}
        self.result.update({col: {"phone_number": text}})

    def clear(self):
        self.result.clear()


class Children:
    def __init__(self):
        self.result = []

    def set_body(self, text):
        self.result = [{
            "object": "block",
            "type": "paragraph",
            "paragraph": {
                "text": [
                    {
                        "type": "text",
                        "text": {
                            "content": text,
                        }
                    }
                ]
            }
        }]



from notion_database.properties import Properties
from notion_database.request import Request


class Database:
    def __init__(self, integrations_token):
        self.properties_list = []
        self.url = 'https://api.notion.com/v1/databases'
        self.result = {}
        self.request = Request(self.url, integrations_token=integrations_token)

    def retrieve_database(self, database_id, get_properties=False):
        self.result = self.request.call_api_get(self.url + "/" + database_id)
        if get_properties:
            self.properties_list.clear()
            for property_value in self.result["properties"].values():
                if property_value["id"] == "title":
                    # property type of the title cannot be changed.
                    continue
                self.properties_list.append(property_value)

    def query_database(self):
        # Not Implemented
        pass

    def run_query_database(self, database_id, body=None):
        """
        for developer
        :param database_id:
        :param body:
        :return:
        """
        if body is None:
            body = {}
        self.result = self.request.call_api_post(self.url + "/" + database_id + "/query", body)

    def find_all_page(self, database_id):
        body = {
            "sorts": []
        }
        self.result = self.request.call_api_post(self.url + "/" + database_id + "/query", body)

    def list_databases(self, page_size=100):
        url = self.url + f"?page_size={str(page_size)}"
        self.result = self.request.call_api_get(url)

    def create_database(self, page_id, title, properties=None):
        if properties is None:
            properties = Properties()
        properties = properties
        body = {
            "parent": {
                "type": "page_id",
                "page_id": page_id
            },
            "title": [
                {
                    "type": "text",
                    "text": {
                        "content": title,
                        "link": None
                    }
                }
            ],
            "properties": properties.result
        }
        self.result = self.request.call_api_post(self.url, body)

    def update_database(self, database_id, title=None, remove_properties=None, add_properties=None):
        if add_properties is None:
            add_properties = Properties()
        body = {
            "properties": {}
        }
        if title:
            body["title"] = [
                {
                    "type": "text",
                    "text": {
                        "content": title,
                        "link": None
                    }
                }
            ]
        if remove_properties:
            body["properties"].update({i["id"]: None for i in remove_properties})
            self.result = self.request.call_api_patch(self.url + "/" + database_id, body)
            body["properties"] = {}
        if add_properties:
            body["properties"].update(add_properties.result)
            self.result = self.request.call_api_patch(self.url + "/" + database_id, body)
            body["properties"] = {}



import json

import requests

from notion_database import NOTION_VERSION


class Request:
    def __init__(self, url, integrations_token):
        self.NOTION_KEY = integrations_token
        self.NOTION_VERSION = NOTION_VERSION
        self.HEADER = {"Authorization": f"Bearer {self.NOTION_KEY}",
                       "Content-Type": "application/json",
                       "Notion-Version": self.NOTION_VERSION}
        self.url = url

    def call_api_post(self, url, body):
        r = requests.post(url, data=json.dumps(body), headers=self.HEADER).json()
        return r

    def call_api_get(self, url):
        r = requests.get(url, headers=self.HEADER).json()
        return r

    def call_api_patch(self, url, body):
        r = requests.patch(url, data=json.dumps(body), headers=self.HEADER).json()
        return r



import logging

from notion_database.properties import Properties, Children
from notion_database.request import Request

LOGGER = logging.getLogger("Notion-Database")


class Page:
    def __init__(self, integrations_token):
        self.url = 'https://api.notion.com/v1/pages'
        self.result = {}
        self.request = Request(self.url, integrations_token=integrations_token)

    def retrieve_page(self, page_id):
        self.result = self.request.call_api_get(self.url + "/" + page_id)

    def create_page(self, database_id, properties=None, children=None):
        if children is None:
            children = Children()
        if properties is None:
            properties = Properties()
        properties = properties
        children = children
        body = {
            "parent": {
                "database_id": database_id
            },
            "properties": properties.result,
            "children": children.result
        }
        self.result = self.request.call_api_post(self.url, body)

        self.check_field()

    def update_page(self, page_id, properties=None):
        if properties is None:
            properties = Properties()
        body = {
            "properties": properties.result,
        }
        self.result = self.request.call_api_patch(self.url + "/" + page_id, body)

        self.check_field()

    def archive_page(self, page_id, archived):
        body = {
            "archived": archived,
        }
        self.result = self.request.call_api_patch(self.url + "/" + page_id, body)

        self.check_field()

    def check_field(self):
        if self.result["object"] == "error":
            LOGGER.error(self.result["message"])
            raise ValueError(self.result["code"])
